/*
    fractal flame generation package
    Copyright (C) 1992, 1993  Scott Draves <spot@cs.cmu.edu>

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/

#include "libifs.h"
#include "cmap.h"

#define argi(s,d)   ((ai = getenv(s)) ? atoi(ai) : (d))
#define args(s,d)   ((ai = getenv(s)) ? ai : (d))

main(argc, argv)
     int argc;
     char **argv;
{
  char s[4000], *ai;
  char *prefix = args("prefix", "default");
  int first_frame = argi("begin", 0);
  int last_frame = argi("end", 100);
  int nstrips = argi("nstrips", 1);
  int dtime = argi("dtime", 1);
  int do_fields = argi("fields", 0);
  int time;
  unsigned char *image;
  control_point templ;
  control_point *cps = (control_point *) malloc(sizeof(control_point) * 300);
  char *ss;
  int c, i, ncps = 0;
  
  /* init the template */
  templ.corner[0] = -1.0;
  templ.corner[1] = -1.0;
  templ.pixels_per_unit = 50;
  templ.width = 100;
  templ.height = 100;
  templ.spatial_oversample = 1;
  templ.gamma = 2.0;
  templ.contrast = 1.0;
  templ.brightness = 1.0;
  templ.spatial_filter_radius = 0.5;
  templ.sample_density = 50;
  templ.nbatches = 10;
  templ.white_level = 200;
  templ.cmap_inter = 0;
  
  if (getenv("template")) {
    FILE *template = fopen(getenv("template"), "r");
    fprintf(stderr, "reading template\n");
    i = 0;
    ss = s;
    do {
      c = getc(template);
      if (EOF == c)
	goto done_reading_template;
      s[i++] = c;
    } while (';' != c);
    
  done_reading_template:
    fclose(template);
    parse_control_point(&ss, &templ);
  }
  
  /* read in all the cps from in */
  fprintf(stderr, "reading in\n");
  if (1) {
    FILE *in = fopen(getenv("in"), "r");
    if (NULL == in)
      in = stdin;
    while (1) {
      i = 0;
      ss = s;
      do {
	c = getc(in);
	if (EOF == c)
	  goto done_reading;
	s[i++] = c;
      } while (';' != c);
      cps[ncps] = templ;
      get_cmap(cmap_random, cps[ncps].cmap, 256);
      parse_control_point(&ss, cps + ncps);
      ncps++;
    }
  }
 done_reading:
  
  image = (unsigned char *) malloc(3 * 256 * 256);
  
  for (time = first_frame; time <= last_frame; time += dtime) {
    int strip, x, y;
    control_point cp;
    interpolate(cps, ncps, (real) time, &cp);
    for (x = 0; x < 256; x++)
      for (y = 0; y < 256; y++) {
	int c;
	unsigned char *p = (unsigned char *) image + 3 * (x * 256 + y);
	for (c = 0; c < 3; c++) {
	  real v = cp.cmap[x][c];
	  if (v < 0.0)
	    p[c] = 0;
	  else if (v > 1.0)
	    p[c] = 255;
	  else
	    p[c] = v * 255.0;
	}
      }
    if (1) {
      FILE *fp;
      char name[100];
      sprintf(name, "%s.%04d.ppm", prefix, time);
      fp = fopen(name, "w");
      fprintf(fp, "P6\n");
      fprintf(fp, "%d %d\n255\n", 256, 256);
      fwrite(image, 1, 3 * 256 * 256, fp);
      fclose(fp);
    }
  }
}
