/*
    fractal flame generation package
    Copyright (C) 1992, 1993  Scott Draves <spot@cs.cmu.edu>

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/

#include "rect.h"
#include <pvm3.h>
#define argi(s,d)   ((ai = getenv(s)) ? atoi(ai) : (d))
#define args(s,d)   ((ai = getenv(s)) ? ai : (d))

main(argc, argv)
   int argc;
   char **argv;
{
   char s[4000], *ai, fname[50];
   int first_frame = argi("begin", 0);
   int nstrips = argi("nstrips", 1);
   int dtime = argi("dtime", 1);
   int do_fields = argi("fields", 0);
   int time, npix;
   unsigned char *image, *Y, *U, *V;
   control_point templ;
   control_point *cps = (control_point *) malloc(sizeof(control_point) * 300);
   char *ss;
   int c, i, ncps = 0;
   frame_spec f;
   int medoframes = 0;

   /* init the template */
   templ.corner[0] = -1.0;
   templ.corner[1] = -1.0;
   templ.pixels_per_unit = 50;
   templ.width = 100;
   templ.height = 100;
   templ.spatial_oversample = 1;
   templ.gamma = 2.0;
   templ.contrast = 1.0;
   templ.brightness = 1.0;
   templ.spatial_filter_radius = 0.5;
   templ.sample_density = 50;
   templ.nbatches = 10;
   templ.white_level = 200;
   templ.cmap_inter = 0;

   if (getenv("template")) {
      FILE *template = fopen(getenv("template"), "r");
      /* fprintf(stdout, "reading template %s\n", getenv("template")); */
      i = 0;
      ss = s;
      do {
	 c = getc(template);
	 if (EOF == c)
	    goto done_reading_template;
	 s[i++] = c;
      } while (';' != c);
	 
    done_reading_template:
      fclose(template);
      parse_control_point(&ss, &templ);
   }

   /* read in all the cps from in */
   /* fprintf(stdout, "reading in %s\n", getenv("in") ? getenv("in") :  "stdin"); */
   if (1) {
      FILE *in = fopen(getenv("in"), "r");
      if (NULL == in)
	 in = stdin;
      while (1) {
	 i = 0;
	 ss = s;
	 do {
	    c = getc(in);
	    if (EOF == c)
	       goto done_reading;
	    s[i++] = c;
	 } while (';' != c);
	 cps[ncps] = templ;
	 get_cmap(cmap_random, cps[ncps].cmap, CMAP_SIZE);
	 parse_control_point(&ss, cps + ncps);
	 ncps++;
      }
   }
 done_reading:
   
   f.temporal_filter_radius = 0.5;
   f.cps = cps;
   f.ncps = ncps;
   npix = cps[0].width * cps[0].height;
   image = (unsigned char *) malloc(3 * npix);
   Y = (unsigned char *) malloc(npix);
   U = (unsigned char *) malloc(npix / 4);
   V = (unsigned char *) malloc(npix / 4);

   (void) pvm_mytid();
   while (1) {
      int strip, frame_index, source_tid;
      control_point cp;
      int bufid;
      /* fprintf(stdout, "waiting for request\n"); */
      bufid = pvm_recv(-1, -1);
      if (1) {
	 int nbytes, tag;
	 if (0 > pvm_bufinfo(bufid, &nbytes, &tag, &source_tid)) {
	    pvm_perror("pvm_bufinfo failed");
	    exit(1);
	 }
	 if (0 != tag) {
	    fprintf(stderr, "unknown tag: %d\n", tag);
	    exit(1);
	 }
      }
      pvm_upkint(&frame_index, 1, 1);
      time = first_frame + dtime * frame_index;
      medoframes++;
      fprintf(stderr, "frame %d time %d medoframes %d\n",
	      frame_index, time, medoframes);
      for (strip = 0; strip < nstrips; strip++) {
	 unsigned char *strip_start =
	    image + cps[0].height * strip * cps[0].width * 3;
	 cps[0].corner[1] = -1.0 + 2.0 * (real) strip / nstrips; /* eek */

	 f.time = (real) time;
	 if (do_fields) {
	    render_rectangle(&f, strip_start, cps[0].width, field_odd);
	    f.time += 0.5;
	    render_rectangle(&f, strip_start, cps[0].width, field_even);
	 } else {
	    render_rectangle(&f, strip_start, cps[0].width, field_both);
	 }
      }
      if (1) {
	 /* convert from rgb to yuv and filter the color
	    portion of the signal down by 2 in x and y */
	 int i, x, y, w, h, ww;
	 unsigned char *p;
	 for (x = 0; x < npix; x++) {
	    p = image + 3 * x;
	    Y[x] = (19595 * p[0] + 38469 * p[1] +  7471 * p[2]) >> 16;
	 }
	 ww = cps[0].width;
	 w = ww/2;
	 h = cps[0].height/2;
	 i = 0;
	 for (y = 0; y < h; y++)
	    for (x = 0; x < w; x++) {
	       p = image + 6 * (x + y * w * 2);
	       U[i] = ((-11056 * p[0] - 21712 * p[1] + 32768 * p[2] +
			-11056 * p[3] - 21712 * p[4] + 32768 * p[5] +
			-11056 * p[0+ww] - 21712 * p[1+ww] + 32768 * p[2+ww] +
			-11056 * p[3+ww] - 21712 * p[4+ww] + 32768 * p[5+ww])
		       >> 18) + 128;
	       V[i] = ((32768 * p[0] - 27440 * p[1] - 5328 * p[2] +
			32768 * p[3] - 27440 * p[4] - 5328 * p[5] +
			32768 * p[0+ww] - 27440 * p[1+ww] - 5328 * p[2+ww] +
			32768 * p[3+ww] - 27440 * p[4+ww] - 5328 * p[5+ww])
		       >> 18) + 128;
	       i++;
	    }
	    
	 /* fprintf(stdout, "sending frame %d time %d\n", frame_index, time); */
	 pvm_initsend(PvmDataDefault);
	 pvm_pkint(&frame_index, 1, 1);
	 pvm_pkbyte(Y, npix, 1);
	 pvm_pkbyte(U, npix / 4, 1);
	 pvm_pkbyte(V, npix / 4, 1);
	 pvm_send(source_tid, 0);
      }
   }
}

